# frozen_string_literal: true
require 'datadog/statsd'

STATSD_HOST = ENV['STATSD_HOST'] || 'localhost'
STATSD_PORT = ENV['STATSD_PORT'] || 8125
CLUSTER_NAME = ENV['CLUSTER_NAME'] || 'unspecified'
STATSD_POOL_SIZE = SERVICE_TASK == 'task-worker' ? 25 : 10
SIDEKIQ_INSTANCE_ID = ENV['SIDEKIQ_INSTANCE_ID'] || 'unspecified'

class ImportRegistry
  register 'statsd_new' do
    runtime = ImportRegistry.resolve('runtime')
    base_tags = %W(env:#{runtime.env} cluster_name:#{CLUSTER_NAME} sidekiq_instance:#{SIDEKIQ_INSTANCE_ID})

    namespace = "cht.svc.#{SERVICE_NAME}"
    if SERVICE_TASK != 'monitoring'
      # For monitoring, we ignore the task from namespace so we can put metrics anywhere
      namespace = "#{namespace}.#{SERVICE_TASK}"
    end

    ::Datadog::Statsd.new(STATSD_HOST, STATSD_PORT, namespace: namespace, tags: base_tags)
  end
  register 'statsd' do
    # We need to use the simulated connection pool because of the way we do dependency
    # injection. If two workers use the same statsd class and try to write to the buffer
    # at the same time, we get a ConcurrencyError. This simulated pool addresses that.
    @statsd_pool ||= ::ConnectionPool::Wrapper.new(size: STATSD_POOL_SIZE, timeout: 5) do
      ImportRegistry.resolve('statsd_new')
    end
  end
end

ImportRegistry.resolve('logger').warn "Statsd initialized with host: #{STATSD_HOST}, PORT: #{STATSD_PORT}"
