require 'kafka'
require 'kafka/datadog'
require 'connection_pool'

at_exit do
  if !$!.nil? && $!.is_a?(SystemExit) && $!.status == 64
    # When Kafka::BufferOverflow is raised we exit without shutting down producer
    # Given the error we most likely already lost connection to kafka
    puts "Pod has been killed due to exit status #{$!.status}"
  else
    begin
      puts 'Shutting down Kafka Producer...'
      ImportRegistry.resolve('kafka_producer').shutdown
    rescue => ex
      $stderr.puts "[at_exit] [kafka_producer] #{ex.class}: #{ex.message}"
    end

    begin
      ImportRegistry.resolve('kafka').close
    rescue => ex
      $stderr.puts "[at_exit] [kafka] #{ex.class}: #{ex.message}"
    end
  end
end

class ImportRegistry
  # TODO: Ideally make this a kafka namespace.
  #       This would mean updating all services and cht-lib-fabric though
  register 'kafka' do
    host = ENV['KAFKA_HOST'] || 'localhost:9092'
    host = "kafka://#{host}"
    @kafka_client ||= ::Kafka.new(
      seed_brokers: [host],
      client_id: SERVICE_TASK,
      logger: ImportRegistry.resolve('logger')
    )
  end

  register 'kafka_producer' do
    @kafka_producer ||= ImportRegistry.resolve('kafka').async_producer(
      # TODO: Use lz4 once the ruby-client supports it
      # http://blog.yaorenjie.com/2017/01/03/Kafka-0-10-Compression-Benchmark/
      compression_codec: :gzip, # for now, optimize for bandwidth and disk usage
      max_queue_size: 25_000, # override default of 1000
      max_buffer_size: 25_000, # override default of 1000
      max_buffer_bytesize: 128.megabytes, # override default of 10MB
      delivery_threshold: 100, # Trigger a delivery once 100 messages have been buffered.
      delivery_interval: 10 # Trigger a delivery every 15 seconds.
    )
  end
end

# This replicates https://github.com/zendesk/ruby-kafka/pull/563
# and should go away when we can upgrade to a released ruby-kafka that
# includes that PR
module Kafka
  module Datadog
    class << self
      def statsd=(statsd)
        clear
        @statsd = statsd
      end
    end
  end
end

Kafka::Datadog.statsd = ImportRegistry.resolve('statsd')
