# frozen_string_literal: true

require 'karafka'
require 'cht-lib-fabric/helpers/topic_mapper'

# Available config options:
# https://github.com/karafka/karafka/blob/master/lib/karafka/setup/config.rb#L60
::Karafka::App.setup do |config|
  # Karafka will auto-discover kafka_hosts based on Zookeeper but we need it set manually
  # to run tests without running kafka and zookeeper
  # NOTE: can remove `.dup` hack once we have ruby-kafka 0.3.16+
  host = (ENV['KAFKA_HOST'] || 'localhost:9092').dup
  host = "kafka://#{host}" unless host.start_with? 'kafka://'
  config.kafka.seed_brokers = [host]
  config.kafka.start_from_beginning = false

  # If $KAFKA_TOPIC_PREFIX is set, prepend this string to all topic names
  # and the client ID.  Only needed in a shored dev cluster.
  prefix = ENV['KAFKA_TOPIC_PREFIX']
  if !prefix.nil? && prefix != ''
    config.topic_mapper = ::Platform::Helpers::TopicMapper.new prefix
    config.client_id = "#{prefix}-#{SERVICE_NAME}"
  else
    config.client_id = SERVICE_NAME
  end

  config.backend = :sidekiq

  if ImportRegistry.resolve('runtime').development? && SERVICE_TASK == 'task-worker'
    # For dev, to limit console output in collectors/processors
    g = Logger.new(STDOUT)
    g.level = :warn
    config.logger = g
  else
    config.logger = ImportRegistry.resolve('logger')
  end
end
