class ImportRegistry
  SSL_ENVS = %w(production staging)

  namespace 'cache' do
    register 'redis' do
      @cache_redis ||= begin
        redis_host = ENV['AWS_REDIS_HOST'] || 'localhost'
        redis_port = ENV['AWS_REDIS_PORT'] || 6379
        # rediss:// is not a typo.
        # Enables SSL connection within `redis` gem.
        redis_uri_scheme = SSL_ENVS.include?(ENV['RACK_ENV']) ? 'rediss://' : 'redis://'
        redis_url = "#{redis_uri_scheme}#{redis_host}:#{redis_port}"
        ImportRegistry.resolve('logger').warn "AWS redis URL: #{redis_url}"

        log = ENV.include?('REDIS_DEBUG') ? ImportRegistry.resolve('logger') : nil

        redis_conn = { url: redis_url,
                       namespace: 'gdas:cht',
                       logger: log,
                       auto_reconnect: true,
                       thread_safe: true,
                       reconnect_attempts: 25 }

        ::Gdas::Clients::Redis.new(redis_conn)
      end
    end

    register 'redis_job_tokens' do
      @cache_redis_job_tokens ||= begin
        redis_host = ENV['JOBTOKEN_REDIS_HOST'] || 'localhost'
        redis_port = ENV['JOBTOKEN_REDIS_PORT'] || 6379
        # rediss:// is not a typo.
        # Enables SSL connection within `redis` gem.
        redis_uri_scheme = SSL_ENVS.include?(ENV['RACK_ENV']) ? 'rediss://' : 'redis://'
        redis_url = "#{redis_uri_scheme}#{redis_host}:#{redis_port}"
        ImportRegistry.resolve('logger').warn "JobTokens redis URL: #{redis_url}"

        log = ENV.include?('REDIS_DEBUG') ? ImportRegistry.resolve('logger') : nil

        redis_conn = { url: redis_url,
                       namespace: 'gdas:tokens:z',
                       logger: log,
                       auto_reconnect: true,
                       thread_safe: true,
                       reconnect_attempts: 25 }

        ::Gdas::Clients::Redis.new(redis_conn)
      end
    end
  end
end

ImportRegistry.resolve('logger').warn "cache.redis: PING #{ImportRegistry.resolve('cache.redis').ping}"
ImportRegistry.resolve('logger').warn "cache.redis_job_tokens (host: #{ENV['JOBTOKEN_REDIS_HOST']}):  PING #{ImportRegistry.resolve('cache.redis_job_tokens').ping}"
