require 'rollbar'

# Need to configure here so that the rack handler works
::Rollbar.configure do |config|
  config.environment = ENV['RACK_ENV']
  config.enabled = !ENV['ROLLBAR_TOKEN'].blank?
  config.access_token = ENV['ROLLBAR_TOKEN']
  config.logger = ImportRegistry.resolve('logger')
  config.populate_empty_backtraces = true

  config.code_version = Gdas::VERSION
  if ImportRegistry.resolve('runtime').development?
    config.branch = `git rev-parse --abbrev-ref HEAD`.chomp rescue nil
  end

  if ImportRegistry.resolve('runtime').development?
    # Need to include point to `ng/cht-svc-gdas`
    config.root = File.absolute_path File.join(__FILE__, '..', '..', '..', '..')
  else
    # Only the `service` folder is included in the container
    config.root = File.absolute_path File.join(__FILE__, '..', '..', '..')
  end
  config.project_gems = %w(collection-framework processing-framework monitoring-framework cht-lib-scheduler cht-lib-fabric)

  # keep in sync with workers in cht-framework-cp
  # cht-framework-cp/collection/lib/cloud_health/collection/task.rb
  config.sidekiq_threshold = 2

  truncation_handler = proc do |options|
    begin
      # TODO: Add `Rollbar.scope!(:person => { :id => user_id })` to context classes
      customer_id = options.dig(:extra, :customer_id)
      if customer_id
        options[:payload]['data'][:person] = { id: customer_id }
        name = ::Customer.where(id: customer_id).pluck(:name).first rescue nil
        options[:payload]['data'][:person][:username] = name unless name.blank?
      end

      ex_msg = options[:payload]['data'][:body][:trace][:exception][:message] rescue nil
      if ex_msg && (ex_msg.include?('access_key') || ex_msg.include?('secret_key'))
        options[:payload]['data'][:body][:trace][:exception][:message].gsub!(/`access_key` = x?'[A-Za-z0-9]+'/, '`access_key` = REDACTED')
        options[:payload]['data'][:body][:trace][:exception][:message].gsub!(/`secret_key` = x?'[A-Za-z0-9]+'/, '`secret_key` = REDACTED')
      end

      # Attempt to strip out the payload of the sidekiq job since they are generally massive
      # and cause rollbar to fail.
      # `Failsafe from rollbar-gem. Could not send payload due to it being too large after truncating attempts.`
      if options[:payload]['data'][:request] && options[:payload]['data'][:request][:params]
        params = options[:payload]['data'][:request][:params]
        max_length = ::Rollbar::Truncation::MAX_PAYLOAD_SIZE * 0.48 # Max is 512KB, use ~251KB
        if params[:jobstr] && params[:jobstr].length > max_length
          args = params[:job]['args']
          offset = args[0].is_a?(::Hash) ? 0 : 1
          key = args[offset]['payload'] ? 'payload' : :payload
          if args[offset][key]
            args[offset][key] = "REDACTED DUE TO SIZE. Original payload size: #{params[:jobstr].length}"
            params[:jobstr] = ::JSON.generate params[:job]
          end
        end
      end
    rescue => ex
      # Ignore if `options` doesn't conform to the job structure that we are looking for
      $stderr.puts "#{ex.class}: #{ex.message}"
      $stderr.puts ex.backtrace
    end
  end

  config.transform << truncation_handler
end

class ImportRegistry
  register 'alert' do
    @alert ||= ::Platform::Handlers::Alert.new
  end
end
