class ImportRegistry
  register 'sidekiq_instance_mgr', memoize: true do
    logger=ImportRegistry.resolve('logger')
    cache = ImportRegistry.resolve('cache.redis')

    # this is a helm-generated list with the format: '[sidekiq1 sidekiq2 sidekiq3]'
    instances_prop=ENV['SIDEKIQ_INSTANCE_IDS'] || '[sidekiq]'
    sidekiq_instances=instances_prop[1..instances_prop.length-2].split(' ')

    logger.warn "INIT sidekiq_instance_mgr, initializing sidekiq configs from instance IDS: #{sidekiq_instances}"
    instance_config_map={}
    sidekiq_instances.each do |sid|
      redis_host=ENV["#{sid.upcase}_REDIS_HOST"] || 'localhost'
      redis_port=ENV["#{sid.upcase}_REDIS_PORT"] || '6379'

      # Get shard assignments from cache. If not found, fall back to environment settings, then save
      config_key=::CloudHealth::JobManagement::Sidekiq::SidekiqInstanceManager.shard_assignment_cache_key(sid)
      db_shard_config=cache.get(config_key)
      if db_shard_config
        logger.warn "Found DB shard config in redis cache for instance: #{sid}: shards: #{db_shard_config}"
      else
        db_shard_config=ENV["#{sid.upcase}_ASSET_DB_SHARDS"]
        if db_shard_config && !db_shard_config.empty?
          logger.warn "Shard config not found in cache for instance: #{sid}, read from env: #{db_shard_config}, saving to cache..."
          cache.set(config_key, db_shard_config)
        else
          logger.warn "Shard config not found in cache or env for instance: #{sid}, will auto-assign"
        end
      end

      instance_config=::CloudHealth::JobManagement::Sidekiq::SidekiqInstanceConfig.new(sid, redis_host, redis_port, db_shard_config)
      instance_config_map[sid]=instance_config
    end

    # Initialize with the config map and a default instance ID
    ::CloudHealth::JobManagement::Sidekiq::SidekiqInstanceManager.new(sidekiq_instances,
                                                                    instance_config_map,
                                                                    ::ImportRegistry.resolve('cache.redis'),
                                                                    ::ImportRegistry.resolve('statsd'),
                                                                    ::ImportRegistry.resolve('logger')
    )
  end
end
