# Extend Sidekiq UI to show Job Token data used for fair queueing
require 'sidekiq/pro/web'
require 'sidekiq/api'

module SidekiqUIExtension
  def self.registered(app)
    # Job Token Management
    app.get('/job_token_summary') do
      @supported_dimensions = Gdas::JobMgmt::TokenLock.supported_owners
      @token_manager = ImportRegistry.resolve('job_token_mgr')
      render(:erb,  File.read("web/job_token_summary.erb"))
    end
    app.get('/job_token_details') do
      @token_manager= ImportRegistry.resolve('job_token_mgr')
      @dimension = params[:dim]
      @showall = params[:showall] ? true : false
      render(:erb,  File.read("web/job_token_details.erb"))
    end
    app.post('/resetkeys') do
      @token_manager= ImportRegistry.resolve('job_token_mgr')
      @dimension = params['dim']
      ImportRegistry.resolve('logger').debug "handling resetkeys request, dimension: #{@dimension}"
      @token_manager.reset_keys(@dimension)
      redirect "#{root_path}job_token_details?dim=#{@dimension}"
    end

    # Sidekiq Instance Management
    app.get('/instance_config') do
      @instance_manager = ImportRegistry.resolve('sidekiq_instance_mgr')
      @show_rec = params[:recommend]
      @result_msg = params[:result_msg]
      render(:erb,  File.read("web/sidekiq_instance_config.erb"))
    end
    app.post('/save_instance_config') do
      @instance_manager= ImportRegistry.resolve('sidekiq_instance_mgr')
      recommended_config = params[:recommended_config]
      result_msg=@instance_manager.save_recommended_configuration(recommended_config)
      redirect "#{root_path}instance_config?result_msg=#{result_msg}"
    end
    app.post('/clear_cached_config') do
      @instance_manager= ImportRegistry.resolve('sidekiq_instance_mgr')
      result_msg=@instance_manager.clear_cached_config
      redirect "#{root_path}instance_config?result_msg=#{result_msg}"
    end

    # Queue Actions
    app.get('/queue_actions') do
      @instance_manager = ImportRegistry.resolve('sidekiq_instance_mgr')
      @result_msg = params[:result_msg]
      render(:erb,  File.read("web/queue_actions.erb"))
    end
    app.post('/queue_action_purge') do
      @instance_manager = ImportRegistry.resolve('sidekiq_instance_mgr')
      sidekiq_instances = @instance_manager.get_current_config
      instance_id = params['instance_id']
      queue_name = params['queue_name']

      result_msg = "Invalid or empty queue." # Default

      instance = sidekiq_instances[instance_id]
      if instance
        ::Sidekiq::Client.via(instance.redis_conn_pool) do
          queue = Sidekiq::Queue.new(queue_name)
          if queue.size > 0
            ImportRegistry.resolve('logger').warn "Purging queue #{queue_name} on instance #{instance_id}. Current size: #{queue.size}."
            queue.clear
            result_msg = "Queue '#{queue_name}' cleared on '#{instance_id}'."
          end
        end
      end

      redirect "#{root_path}queue_actions?result_msg=#{result_msg}"
    end
  end
end

# here we instruct Sidekiq to take our UI extension onboard
::Sidekiq::Web.register SidekiqUIExtension

# right hand of the equation should be equal to the path you specified
# in registered() method
::Sidekiq::Web.tabs['Job Tokens'] = 'job_token_summary'
::Sidekiq::Web.tabs['Instance Configuration'] = 'instance_config'
::Sidekiq::Web.tabs['Queue Actions'] = 'queue_actions'
